"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Spinners = require("cli-spinners");
const logFigures = require("figures");
const chalk_1 = require("chalk");
const task_manager_1 = require("./task-manager");
const wcwidth = require("wcwidth");
const stripAnsi = require("strip-ansi");
class CliLoading {
    constructor(props) {
        this._loadingFrameMap = new Map();
        this.linesToClear = 0;
        this.props = Object.assign({ indent: 3, symbolMap: {}, grayState: null, stream: process.stderr }, props);
    }
    count(uid) {
        let count = this._loadingFrameMap.get(uid) || 0;
        this._loadingFrameMap.set(uid, count + 1);
        return count;
    }
    renderDepsTree(depsTree, output = []) {
        let indent = Array(depsTree.depth * this.props.indent)
            .fill(' ')
            .join('');
        let frames = Spinners.dots.frames;
        let symbol = Object.assign({ [task_manager_1.TaskState.waiting]: chalk_1.default.gray(logFigures.ellipsis), [task_manager_1.TaskState.pending]: chalk_1.default.blueBright(logFigures.arrowRight), [task_manager_1.TaskState.loading]: chalk_1.default.cyan(frames[this.count(depsTree.uid) % frames.length]), [task_manager_1.TaskState.succeeded]: chalk_1.default.green(logFigures.tick), [task_manager_1.TaskState.failed]: chalk_1.default.red(logFigures.cross), [task_manager_1.TaskState.skipped]: chalk_1.default.yellow(logFigures.info) }, this.props.symbolMap)[depsTree.state];
        let color = (this.props.grayState || [task_manager_1.TaskState.waiting]).indexOf(depsTree.state) >= 0
            ? f => chalk_1.default.gray(f)
            : f => f;
        let skipped = depsTree.state === task_manager_1.TaskState.skipped;
        output.push(`${indent}${symbol} ${color(depsTree.task.name)}${skipped ? chalk_1.default.gray(` [skipped]`) : ''}`);
        for (const child of depsTree.asyncDeps.concat(depsTree.syncDeps)) {
            this.renderDepsTree(child, output);
        }
        return output;
    }
    render() {
        let columns = this.props.stream.columns || 80;
        let rows = this.props.stream.rows || Infinity;
        let output = this.renderDepsTree(this.props.depsTree);
        let isFirstRendering = this.linesToClear === 0;
        output.push('');
        let outputLineCounts = output.map(line => Math.max(1, Math.ceil(wcwidth(stripAnsi(line)) / columns)));
        this.clear();
        this.linesToClear = 0;
        for (let i = outputLineCounts.length - 1; i >= 0; i--) {
            const count = outputLineCounts[i];
            this.linesToClear += count;
            if (this.linesToClear > rows) {
                this.linesToClear -= count;
                if (!isFirstRendering) {
                    output = output.slice(i + 1);
                }
                break;
            }
        }
        this.props.stream.write(output.join('\n'));
    }
    // Adopted from https://github.com/sindresorhus/ora/blob/bbc82a44884b23f1787d91f95b2d3f93afe653e3/index.js#L74
    clear() {
        if (!this.props.stream.isTTY) {
            return this;
        }
        for (let i = 0; i < this.linesToClear; i++) {
            if (i > 0) {
                this.props.stream.moveCursor(0, -1);
            }
            this.props.stream.clearLine();
            this.props.stream.cursorTo(0);
        }
        this.linesToClear = 0;
        return this;
    }
    start() {
        if (this.id)
            return;
        this.id = setInterval(() => {
            this.render();
        }, 100);
    }
    stop() {
        this.id && clearInterval(this.id);
        this.id = undefined;
        this.render();
    }
}
exports.CliLoading = CliLoading;
//# sourceMappingURL=cli-loading.js.map