"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
let objUid = 0;
let objUidMap = new WeakMap();
/** @internal */
function getType(key) {
    const t = Object.prototype.toString.call(key);
    return t.slice(8, -1).toLowerCase();
}
exports.getType = getType;
/** @internal */
function hashAny(key) {
    switch (getType(key)) {
        case 'undefined':
        case 'null':
        case 'boolean':
        case 'number':
        case 'regexp':
            return key + '';
        case 'date':
            return '📅' + key.getTime();
        case 'string':
            return '📝' + key;
        case 'array':
            return '🔗' + key.map(k => hashAny(k)).join('⁞');
        case 'object':
            return key.constructor.name + JSON.stringify(key, (k, v) => {
                if (!k)
                    return v;
                return hashAny(v);
            });
        default:
            let uid = objUidMap.get(key);
            if (!uid) {
                uid = ++objUid;
                objUidMap.set(key, uid);
            }
            return '⭕️' + uid;
    }
}
exports.hashAny = hashAny;
exports.sleep = (ms) => {
    return new Promise(res => setTimeout(res, ms));
};
function throttle(cb, ms) {
    let timer;
    let newCb = (...args) => {
        timer && clearTimeout(timer);
        timer = setTimeout(cb, ms, ...args);
    };
    return newCb;
}
exports.throttle = throttle;
exports.Is = {
    defed(v) {
        return typeof v !== 'undefined' && v !== null;
    },
    str(v) {
        return typeof v === 'string';
    },
    bool(v) {
        return typeof v === 'boolean';
    },
    fn(v) {
        return typeof v === 'function';
    },
    obj(v) {
        return v && typeof v === 'object';
    },
    num(v) {
        return typeof v === 'number';
    }
};
function defaults(...args) {
    let [val, ...defaultVals] = args;
    if (exports.Is.defed(val))
        return val;
    if (defaultVals.length === 0) {
        return val;
    }
    return defaults(...defaultVals);
}
exports.defaults = defaults;
const nsSet = new Set();
/**
 * Generate unique task names with namespaces via object tree.
 * @param obj
 * @param ns Namespace
 * @param sep Separator for namespaces
 * @example
 *  ```ts
 * /** N.aa.bb => 'aa:bb' *\/
 * const N = namespacify({
 *   aa: { bb: { cc: '' } },
 *   cc: '',
 *   ff: ''
 * })
 *
 * /* =>
 *    { aa: { bb: { cc: 'aa:bb:cc' } },
 *      cc: 'cc',
 *      ff: 'ff' } *\/
 *  ```
 */
function namespacify(obj, ns = '', sep = ':') {
    if (nsSet.has(obj))
        return obj;
    for (const key in obj) {
        const val = obj[key] || key;
        if (exports.Is.str(val)) {
            obj[key] = `${ns}${ns && sep}${val}`;
        }
        else if (exports.Is.obj(val)) {
            namespacify(val, `${ns}${ns && sep}${key}`, sep);
        }
    }
    nsSet.add(obj);
    return obj;
}
exports.namespacify = namespacify;
function namespace(ns, fn) {
    fn(namespacify(ns));
}
exports.namespace = namespace;
exports.DefaultLogFile = path_1.join(process.cwd(), 'foy.log');
//# sourceMappingURL=utils.js.map